import math
import pyaudio
from struct import unpack
from random import shuffle, sample



opzioni = {
	"format": pyaudio.paFloat32,
	"channels": 2,
	"rate": 96000,
	"input": True,
	"frames_per_buffer": 1024
}


class primes:

	def prime(n, k=64):
		"""
		Probabilistic primality test Miller-Rabin.
		Return True if n is probably prime otherwise return False.
		The probability that it is wrong is 2^-2k
		"""
		if n < 2 or n % 2 == 0:
			return False
		if n == 2:
			return True

		r, s = 0, n - 1
		while s % 2 == 0:
			r += 1
			s //= 2

		for i in range(k):
			a = random.randrange(2, n)
			x = pow(a, s, n)
			if x == 1 or x == n - 1:
				continue
			for i in range(r - 1):
				x = pow(x, 2, n)
				if x == n - 1:
					break
			else:
				return False
		return True

	def primes(p, n):
		"""
		Returns the n primes subsequent p.
		[Generator]
		"""
		if p == 2:
			n -= 1
			yield p
		if p % 2 == 0:
			p += 1
		i = 0
		while True:
			if i == n:
				break
			if primes.prime(p):
				i += 1
				yield p
			p += 2


class generator:

	def _nums(l, n):
		"""
		Split bytes l into n 32-bit unsigned int.
		[Generator]
		"""
		for i in range(0, len(l), n):
			chunk = l[i:i+n]
			if any(chunk): # throw away initial zeros
				yield unpack('>L', chunk)[0]

	def random(a, b, n):
		"""
		Returns n random 32-bit float in the range [a, b).
		[Generator]
		"""
		audio = pyaudio.PyAudio()
		raw_data = audio.open(**opzioni).read(
			opzioni["rate"] // opzioni["frames_per_buffer"] * 
			int(10*math.log(n)))
		audio.terminate()
		
		sequence = generator._nums(raw_data, 4)
		sequence = sample(list(sequence), n)
		m = max(sequence)

		return map(lambda x: a + (b-a) * (x/m), sequence)

	def onetimepad(l):
		"""
		Return an l-uple of random integers for a one-time-pad.
		"""
		return tuple(map(int, generator.random(0, 2*16, l)))
