from .numbers import *

max = 2 ** 16


class encrypt:

	def __init__(self, value):
		"""
		Initialization
		Provide a string to encrypt.
		"""
		self.string = value

	def rot13(self):
		"""
		Cifrario ROT13
		"""
		rotation = str.maketrans(
			"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ",
			"nopqrstuvwxyzabcdefghijklmNOPQRSTUVWXYZABCDEFGHIJKLM")
		return str.translate(self.string, rotation)

	def vigenere(self, worm):
		"""
		vigenère Chiper
		Provide an alphabetical key length less than the string (worm).
		"""
		encrypted = []
		for i, character in enumerate(self.string):
			encrypted.append(
				chr((ord(character) + ord(worm[i % len(worm)])) % max))
		return "".join(encrypted)

	def godel(self, prime=2):
		"""
		Gödel number
		Provide a prime.
		"""
		encrypted = 1
		sequence = primes.primes(prime, len(self.string))
		for i, character in enumerate(self.string):
			encrypted *= next(sequence) ** ord(character)
		return encrypted

	def vernam(self, pad=()):
		"""
		Vernam Cipher
		A one-time-pad will be generated automatically if not provided.
		"""
		encrypted = []
		if pad is ():
			pad = generator.onetimepad(len(self.string))
		for i, character in enumerate(self.string):
			encrypted.append(
				chr((ord(character) + pad[i]) % max))
		return "".join(encrypted), pad


class decrypt:

	def __init__(self, value):
		"""
		Initialization
		Provide a string or a number (Gödel) to decrypt.
		"""
		if str(value).isnumeric():
			self.numero = value
		else:
			self.string = value

	def rot13(self):
		"""
		ROT13 Cipher
		"""
		rotation = str.maketrans(
			"nopqrstuvwxyzabcdefghijklmNOPQRSTUVWXYZABCDEFGHIJKLM",
			"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ")
		return str.translate(self.string, rotation)

	def vigenere(self, worm):
		"""
		Vigenere Cipher
		Provide the key used to encrypt the string (worm).
		"""
		decrypted = []
		for i, character in enumerate(self.string):
			decrypted.append(
				chr((ord(character) - ord(worm[i % len(worm)])) % max))
		return "".join(decrypted)

	def godel(self, n, prime=2):
		"""
		Gödel number
		Specify a starting prime and the length of the string.
		"""
		decrypted = []
		for i in primes.primes(prime, n):
			r, s = 0, self.numero
			while s % i == 0:
				r += 1
				s //= i
			decrypted.append(chr(r % max))
		return "".join(decrypted)

	def vernam(self, pad):
		"""
		Vernam Cipher
		Provide the one-time-pad used to encrypt the string.
		"""
		encrypted = []
		for i, character in enumerate(self.string):
			encrypted.append(
				chr((ord(character) - pad[i]) % max))
		return "".join(encrypted)
